@@ -1,3 +1,20 @@
+# Copyright 2015-2019 Venelin Mitov
+#
+# This file is part of POUMM.
+#
+# POUMM is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# POUMM is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with POUMM  If not, see <http://www.gnu.org/licenses/>.
+
 #' @rdname OU
 #' @title Distribution of an Ornstein-Uhlenbeck Process at Time \eqn{t}, Given 
 #'   Initial State at Time \eqn{0}
@@ -485,7 +502,7 @@ gPOUMM <- function(z, tree, g0, alpha, theta, sigma, sigmae) {
     mu.g.poumm <- mu.g
   }
 
-  if(class(V.g.poumm)=='try-error') {
+  if(inherits(V.g.poumm, 'try-error')) {
     warning(V.g.poumm)
   } 
   list(V.g = V.g, V.g_1 = V.g_1, mu.g = mu.g, V.e = V.e, V.e_1 = V.e_1,
